#include <stdio.h>
#include <string.h>                                                          
#include <math.h>
#include <stdlib.h>
#include "arms.h"                                                            
#define MATHLIB_STANDALONE 1
#include <Rmath.h>
#include "global.h"
#include <gsl/gsl_integration.h>


double Hsold(double *lam, beta_vector b, double *tint,  double gamma, 
	  int obs, double *c, int lamind){

  /* tint = time points defining intervals for hazard step function */
  /* c =  times where measurements are taken with failure time sorted in.*/
  /* t = where measurements are taken */
  /* lamind = if =-1, summing over all intervals.  otherwise just one interval
   we do this for use when sampling lambdas.*/
  /* obs = number of longitudinal observations on a patient  */



  double  r, l, B;
  int j,k, intsamp=0;
  /* j indexes over observations */
  /* k indexes over hazard intervals */
  
  if(lamind > -1) {
    intsamp=1;
  }
  
  B=0;
  l=c[0];
  l=0;	
  k=1;
  for(j=1;j<(obs);j++){/*  <obs if pobs includes event time, 
			   < obs+1 if pobs does not include event time */
    while(tint[k]<c[j]) {
      r=tint[k];
      if(intsamp==0 || lamind==(k-1)){
	if(r<l){
	  printf("r=%lf < l=%lf\n Make sure your observation times are sorted correctly! j= %d", r,l,j);
	  exit(1);
	}
	B+=lam[k-1]*exp(gamma*traj(c[j-1],b))*(r-l);
      }
      l=r;
      k++;
    }   
    r=c[j];   
    if(intsamp==0 || lamind==(k-1)){
	if(r<l){
	  printf("r=%lf < l=%lf\n Make sure your observation times are sorted correctly! j= %d", r,l,j);
	  exit(1);
	}
      B+=lam[k-1]*exp(gamma*traj(c[j-1],b))*(r-l);
    }
    if(r==tint[k]) k++;   
    l=r;
  }
  return B;
}


double Hsu(double x, void *params) {
  struct HS *par = (struct HS *)params;

  return exp(par->g*(par->b.b0+par->b.b1*x+par->b.b2*x*x));
}

double Hs(double *lam, beta_vector b, double *tint,  double gamma, 
	  int obs, double *c, int lamind){

  double result, error;
  double intres;
  struct HS hs;
  int j;
  int neval;
  int  intsamp=0;
  
  if(lamind > -1) {
    intsamp=1;
  }
  gsl_function F;

  F.function = &Hsu;
  set_beta_vector(&hs.b,b.b0,b.b1,b.b2);

  hs.g=gamma;
  intres=0;
  j=0;
  while((tint[j+1]<c[obs-1])&&j<J){
    if(intsamp==0 || lamind==j){
      hs.lambda=lam[j];
      F.params = &hs;
/*       printf("%d %f %f ",j, tint[j], tint[j+1]); */
      gsl_integration_qng (&F, tint[j], tint[j+1], 1e-10, 1e-7, &result, 
			   &error, &neval); 
      intres+=lam[j]*result;
/*       printf(" %f %f \n",intres, result); */
      result=0;
    }
    j++;
  }
  if(intsamp==0|| lamind==j){
    hs.lambda=lam[j];
    F.params = &hs;
    /*     printf("%d %f %f ",j, tint[j],c[obs-1] ); */
    gsl_integration_qng (&F, tint[j], c[obs-1], 1e-5, 1e-3, &result, 
			 &error, &neval); 

    intres+=lam[j]*result;
  }
  return intres;
}
