#include <gsl/gsl_matrix.h>
#include <gsl/gsl_linalg.h>
#include <gsl/gsl_blas.h>
#include <math.h>
#include <stdio.h>
#include <gsl/gsl_rng.h>
#include <gsl/gsl_randist.h>
#include "global.h"

void set_beta_vector(beta_vector *b, double x1, double x2, double x3){
  b->b0=x1;
  b->b1=x2;
  b->b2=x3;
}

void print_beta_vector(beta_vector b) {
  printf("%lf %lf %lf\n", b.b0, b.b1,b.b2);
}

double bilike(beta_vector bi, struct data *d) {
  /*  This function calculates the log-likelihood of beta0i */
  double tempsum;
  int i, j;
     
  i=d->p->index; /* set the subject whose contribution we are
		    calculating */

  tempsum=0;
  
  for(j=0;j<d->obs[i];j++){
    tempsum+=pow(traj(d->t[i][j],bi),2)-2*d->y[i][j]*traj(d->t[i][j],bi);
  }
  tempsum*= -.5*d->p->tau;

/* evaluating trajectory at the last available measurement before event*/
  if(d->delta[i]==1) tempsum += d->p->gamma*traj(d->c[i][d->pobs[i]-1],bi);
  if(d->p->tint[0]!=0) printf("tint0 <0, i=%d\n",i);
  tempsum -= exp(d->p->alpha*d->trt[i])*
    Hs(d->p->lambda,bi,d->p->tint,d->p->gamma,
       d->pobs[i],&d->c[i][0], -1);

   return tempsum;
}

double phiipost(beta_vector b, void *data1, int i) {
  /*  i = subject who is contributing to log posterior  */
  /*  This function calculates the posterior of phi0|z  */
  struct data *d;
  double t2=0;
  int j;
  
  d=data1;
  
  for(j=0;j<d->obs[i];j++){
    t2+=pow(traj(d->t[i][j],b),2)-2*d->y[i][j]*traj(d->t[i][j],b);
  }
  t2*= -.5*d->p->tau;

  /* evaluating trajectory at the last available measurement */
  if(d->delta[i]==1) t2 +=d->p->gamma*traj(d->c[i][d->pobs[i]-1],b);
  if(d->p->tint[0]!=0) printf("tint0 <0, i=%d\n",i);
  t2 += -exp(d->p->alpha*d->trt[i])*
    Hs(d->p->lambda,b,d->p->tint,d->p->gamma,
       d->pobs[i],&d->c[i][0], -1);

  return t2;
}  


double phi0ipost(double x, void *data1) {
  /*  This function calculates the posterior of phi0|z  */
  struct data *d;
  beta_vector b;
  double tempsum, prior=0;
  int i;
  d=data1;


  tempsum=0;
  for(i=0; i<N; i++){ 
    if(d->p->z[i]==d->p->index){  /* if i is in this cluster  */
      set_beta_vector(&b,x,d->beta1i[i],d->beta2i[i]);
      tempsum+=phiipost(b,d,i);
    }
  }
  
  prior=b.b0*(b.b0*gsl_matrix_get(d->p->V0inv,0,0) +
	      2.0*b.b1*gsl_matrix_get(d->p->V0inv,0,1) +
	      2.0*b.b2*gsl_matrix_get(d->p->V0inv,0,2))-
    2.0*b.b0*(d->p->beta0*gsl_matrix_get(d->p->V0inv,0,0)+
	      d->p->beta1*gsl_matrix_get(d->p->V0inv,0,1)+
	      d->p->beta2*gsl_matrix_get(d->p->V0inv,0,2));


  tempsum += -.5*prior;

  return tempsum;
}                                                                              

double phi1ipost(double x, void *data1) {

  /*  This function calculates the posterior of phi1|z  */
  struct data *d;
  beta_vector b;
  double tempsum, prior=0;
  int i;
  d=data1;

  tempsum=0;
  
  for(i=0;i<N;i++){ 
    if(d->p->z[i]==d->p->index){
      set_beta_vector(&b,d->beta0i[i],x,d->beta2i[i]);
      tempsum+=phiipost(b,d,i);
    }
  }

  prior=b.b1*(b.b1*gsl_matrix_get(d->p->V0inv,1,1) +
	      2.0*b.b0*gsl_matrix_get(d->p->V0inv,1,0) +
	      2.0*b.b2*gsl_matrix_get(d->p->V0inv,1,2)) -
    2*b.b1*(d->p->beta0*gsl_matrix_get(d->p->V0inv,1,0)+
	    d->p->beta1*gsl_matrix_get(d->p->V0inv,1,1)+
	    d->p->beta2*gsl_matrix_get(d->p->V0inv,1,2));
  
  return (tempsum-.5*prior);
}

double phi2ipost(double x, void *beta0_data) {
/*  This function calculates the posterior of phi0|z  */
  struct data *d;
  beta_vector b;
  double tempsum, prior=0;
  int i;
  d=beta0_data;
  b.b2=x;
  tempsum=0;
  
  for(i=0;i<N;i++){ 
    if(d->p->z[i]==d->p->index){
      b.b0=d->beta0i[i];
      b.b1=d->beta1i[i];
      tempsum+=phiipost(b,d,i);
    }
  }
  prior=b.b2*(b.b2*gsl_matrix_get(d->p->V0inv,2,2) +
	      2.0*b.b0*gsl_matrix_get(d->p->V0inv,2,0) +
	      2.0*b.b1*gsl_matrix_get(d->p->V0inv,2,1)) -
    2*b.b2*(d->p->beta0*gsl_matrix_get(d->p->V0inv,2,0)+
	    d->p->beta1*gsl_matrix_get(d->p->V0inv,2,1)+
	    d->p->beta2*gsl_matrix_get(d->p->V0inv,2,2));
  
    tempsum += -.5*prior;
    return tempsum;
}                                                                              


void betasamp(struct data *d , double *phib0, double *phib1, 
	      double *phib2, 
	      int b0par, int k,  gsl_rng * r) {
  /* ************ SAMPLE b0 ******************** */
  /* when we are sampling from the parametric model, the phib* vectors
     are of length N corresponding to the betai's */

  double   ps[3], x[3];  
  int i,j,  signum;
  gsl_vector *mu = gsl_vector_alloc(3);
  gsl_vector *psum = gsl_vector_calloc(3);
  gsl_matrix *sig = gsl_matrix_alloc(3,3);
  gsl_matrix *sig2 = gsl_matrix_alloc(3,3);
  gsl_permutation * P = gsl_permutation_alloc (3);

  /* calculate covariance matrix first */
  gsl_matrix_memcpy(sig,d->p->V0inv); /* sig = V_0{-1} */
  gsl_matrix_scale(sig, (double) k);  /* sig = kV_0{-1} */
  gsl_matrix_add(sig,d->p->C1inv);  /* sig = kV_0^{-1} + C_1^{-1} */
  /* sig is now the inverse covariance matrix for the conditional posterior */


  for(i=0; i<3; i++) ps[i]=0;
  /* ps will be a vector corresponding to the sums of the phi's */
  for(i=0; i<k; i++){
    ps[0]+=phib0[i];
    ps[1]+=phib1[i];
    ps[2]+=phib2[i];
  }

  /* psum will be a vector corresponding to the sums of the phi's */
  gsl_vector_set(psum,0,ps[0]);
  gsl_vector_set(psum,1,ps[1]);
  gsl_vector_set(psum,2,ps[2]);

  gsl_blas_dsymv(CblasUpper,1,d->p->V0inv,psum,0,mu); /* \mu=V_0^{-1}\times
							 \sum \phi_k */

  /* now add prior mean */
  gsl_vector_add(mu,d->p->C0C1i);
  /* psum is now the mean */
  gsl_linalg_LU_decomp(sig, P, &signum); 

  gsl_linalg_LU_invert(sig, P, sig2);  /* sig2 = 
					  (kV_0^{-1} + C_1^{-1})^{-1} */

  gsl_blas_dsymv(CblasUpper,1,sig2,mu,0,psum);/* \mu=(kV_0^{-1}+C_1^{-1})
						 ^{-1}V_0^{-1}\sum\phi_k */
  mvn(psum, sig2, 3, x, r);
  d->p->beta0=x[0];
  d->p->beta1=x[1];
  d->p->beta2=x[2];

  gsl_vector_free(mu);
  gsl_vector_free(psum);
  gsl_matrix_free(sig);
  gsl_matrix_free(sig2);
  gsl_permutation_free(P);
}
