/* Dirichlet process prior joint surv/long model - version 1 */
/* This is the joint model with a mixed effects linear model */
/*   and a survival covariates for treatment.  */

/* Elizabeth Brown */

#include <stdio.h>
#include <string.h>                                                          
#include <math.h>
#include <stdlib.h> 
#include "arms.h"                                                            
#define MATHLIB_STANDALONE 1
#include <Rmath.h>
#include <gsl/gsl_blas.h>
#include <gsl/gsl_matrix.h>
#include <gsl/gsl_rng.h>
#include <gsl/gsl_randist.h>
#include "global.h"

char *in_file;
char *out_file;
char *t_file;
char *bsum_file;
char *l_file;


int printb=0;  /* =1 -> print random effects samples */
int bpar = 0;  /* 0 for nonpar and 1 for par */
int sampalph = 0;  /* 1 if we want to sample alpha */
int samptb = 1;
int spred = 0;
double pi = 3.141593;
double resultf[1], lscale;
void *dummy;

int npsample=4;  /* 1 if using quadapp
		  3 if using nogaps
		  4 if using neal */

double slike(double x, void *data1) {
/*  calculates the log-likelihood of s for subject i conditional on currently 
    sampled parameter values  */
  struct data *d;
  beta_vector bi;
  double tempsum,c[11];
  double tint[J+1];
  int i, j, pobs, sind;
  d=data1;       
  i=d->p->index;
  /* i is the index of the subject we're currently sampling */

  set_beta_vector(&bi, d->beta0i[i],d->beta1i[i],d->beta2i[i]);

  tempsum=0;

  sind=0;
  /* pobs=d->obs[i]; */
  if(x>d->t[i][0]){	
    /* if the time sampled is after the first observation time for the 
       subject */
    for(j=1;j<J;j++){
      if (x>d->p->tint[j]) sind+=1;
      tint[j]=d->p->tint[j];
    }
    
    if(x>d->p->tint[J]) tint[J]=x;
    else tint[J]=d->p->tint[J];
    /* make own set of time points since we are doing predictions. */
    /* do we really want to do that or should we stick to observations */
    for(j=0;j<10;j++){
      c[j]=j*x/11;
    }
    c[10]=x;
    pobs=10;

    

    tempsum += d->p->gamma*traj(x,bi),0;    
    tempsum += -exp(d->p->alpha*d->trt[i])*
      Hs(d->p->lambda,bi,tint,d->p->gamma,pobs,
	 c,-1);
    tempsum+=log(d->p->lambda[sind]);
    return tempsum;
  }
  else {  /* pobs=0 */
    tempsum=(d->p->gamma*traj(0,bi) 
		 -exp(d->p->alpha*d->trt[i])*d->p->lambda[0]*
		 exp(d->p->gamma*bi.b0)*x+log(d->p->lambda[0]));
    return tempsum;
  }
}


double alphapost(double x, void *data1) {
  /* This is the log of the conditional posterior of gamma */
  struct data *d;
  beta_vector b;
  double tempsum;
  int i;
  d=data1;

  tempsum=0;
  for(i=0;i<N;i++){
    set_beta_vector(&b,d->beta0i[i], d->beta1i[i],d->beta2i[i]);
    tempsum+=x*d->delta[i]*d->trt[i];    
    tempsum+= -exp(x*d->trt[i]) * Hs(d->p->lambda,b,d->p->tint,
				     d->p->gamma,d->pobs[i],&d->c[i][0],-1);    
  }
  tempsum+= -.5/d->p->sigmaalpha*(pow(x,2)-2*x*d->p->mualpha);
  return tempsum;
}


double gampost(double x, void *data1) {
  /* This is the log of the conditional posterior of gamma */
  struct data *d;
  beta_vector b;
  double tempsum;
  int i;
  d=data1;
  
  tempsum=0;
  for(i=0;i<N;i++){
    set_beta_vector(&b, d->beta0i[i], d->beta1i[i], d->beta2i[i]);
    
    /* evaluate trajectory at the last available measurement before the event*/
    /* updated to be the event time - 6/26/2002 */

    if(d->delta[i]==1){
      /*  printf("traj=%f %f  %f\n", x,d->c[i][d->pobs[i]-1],traj(d->c[i][d->pobs[i]-1], b)); */ 
      tempsum+=x*traj(d->c[i][d->pobs[i]-1], b); 
    }
    if(d->p->alpha*d->trt[i] == 0){
      tempsum -= Hs(d->p->lambda,b,d->p->tint,x,d->pobs[i],
		    &d->c[i][0],-1);

    }
    else{
      tempsum -= exp(d->p->alpha*d->trt[i])*
	Hs(d->p->lambda, b, d->p->tint,  x, d->pobs[i],
	   &d->c[i][0], -1);
    }
  }
  tempsum+= -.5/d->p->sigmagsq*(pow(x,2)-2*x*d->p->mug);
/*   printf("%f %f\n", x,tempsum); */
  return tempsum;
}


void tau0samp(struct data *d , double *phib0, int b0par, int kb0, 
	      double ab,double bb) {
  double sumbisq, scale, shape;  
  int l;

      sumbisq=0;
      if(b0par==1){ /* PARAMETRIC SAMPLING */
	for (l=0;l<N;l++){
	  sumbisq+=pow(d->beta0i[l]-d->p->beta0,2);
	}
	scale=bb+.5*sumbisq;     
	shape=ab+(double)N*.5;
      }
      if(b0par==0){ /* NONPARAMETRIC SAMPLING */
	for (l=0;l<kb0;l++){
	  sumbisq += (phib0[l]-d->p->beta0)*(phib0[l]-d->p->beta0);
	}
	scale=bb+.5*sumbisq;     
	shape=ab+(double)kb0*.5;
      }
/*      d->p->taub0=rgamma(shape,1/scale);  */
}




void tau1samp(struct data *d , double *phib1, int b1par, int kb1,
	      double ab,double bb) {
  double sumbisq, scale, shape; 
  int l; 
       sumbisq=0;
       if(b1par==1){
	 for (l=0;l<N;l++){
	   sumbisq+=pow(d->beta1i[l]-d->p->beta1,2);
	 }
	 scale=bb+.5*sumbisq;     
	 shape=ab+(double)N*.5;
       }
       
       if(b1par==0){
	 for (l=0;l<kb1;l++){
	   sumbisq+=pow(phib1[l]-d->p->beta1,2);
	 }	 
	 shape=ab+(double)kb1*.5;
	 scale=bb+.5*sumbisq;     
       }       
/*       d->p->taub1=rgamma(shape,1/scale); */
     }
    


void tau2samp(struct data *d , double *phib2, int b2par, int kb2,
	      double ab, double bb) {
  double sumbisq, scale, shape; 
  int l; 
  sumbisq=0;
  if(b2par==1){
    for (l=0;l<N;l++){
      sumbisq+=pow(d->beta2i[l]-d->p->beta2,2);
    }
    shape=ab+(double)N*.5;
    scale=bb+.5*sumbisq; 
  }
  
  if(b2par==0){
    for (l=0;l<kb2;l++){
      sumbisq+=pow(phib2[l]-d->p->beta2,2);
    }
    shape=ab+(double)kb2*.5;
    scale=bb+.5*sumbisq;     
  }
  
/*  d->p->taub2=rgamma(shape,1/scale); */
}


       
void rmbeta0(double *phib0, double *b0init, int ninit, double b0l, 
	     double b0r,
	     struct data *d, 
	     int npoint, int kb0) {
  
  double b0samp[1], qcent[5]={10,20,50,70,80}, xcent[5], b0prev[N], b0convex=1;
  int err, nsamp =1, neval, ncent=5, dometrop=0, j;


  for(j=0;j<kb0;j++){
    d->p->index=j;
    b0prev[j]=phib0[j];
    err = arms(b0init,ninit,&b0l,&b0r,phi0ipost,d, &b0convex,npoint,
	       dometrop,&b0prev[j],b0samp,nsamp,qcent,xcent,ncent,&neval);
    
    if(err>0){
      printf("phi0[j] error code = %d   \n ",err);
      printf("kb0= %d, b0prev[%d] = %lf\n", kb0,j, b0prev[j]);
      exit(1);
    }
    phib0[j]=b0samp[nsamp-1];
  }
}

void rmbeta1(double *phib1, double *b1init, int ninit, double b1l, 
	     double b1r,
	     void *data1, int npoint, int kb1) {
  
  double b1samp[1], qcent[5]={10,20,50,70,80}, xcent[5], b1prev[N], b1convex=1;
  int err, nsamp =1, neval, ncent=5, dometrop=0, j,i;  
  struct data *d;
  d=data1;
  
  for(j=0;j<kb1;j++){
    b1prev[j]=phib1[j];

    if(b1prev[j]>b1r || b1prev[j]<b1l) b1prev[j]=(b1r+b1l)/2;
    
    d->p->index=j;
    err = arms(b1init,ninit,&b1l,&b1r,phi1ipost,d,&b1convex,npoint,
	       dometrop,&b1prev[j],b1samp,nsamp,qcent,xcent,ncent,&neval);
    
    if(err>0){
      printf("phib1 error code = %d\n",err);
      exit(1);
    }
    phib1[j]=b1samp[0]; 
  } 
}

void rmbeta2(double *phib2, double *b2init, int ninit, double b2l, double b2r,
	     void *data1,int npoint, int kb2){

  double b2samp[1], qcent[5]={10,20,50,70,80}, xcent[5], b2prev[N], b2convex=1;  
  int err, nsamp =1, neval, ncent=5, dometrop=1, j,i;  
  struct data *d;
  d=data1;
  for(j=0;j<kb2;j++) {    
    d->p->index=j;
    b2prev[j]=phib2[j]; 
        
    if(b2prev[j]>b2r || b2prev[j]<b2l) b2prev[j]=(b2r+b2l)/2;

    err = arms(b2init,ninit,&b2l,&b2r,phi2ipost,data1,&b2convex,npoint,
	       dometrop,&b2prev[j],b2samp,nsamp,qcent,xcent,ncent,&neval);
    
    if(err>0){
      printf("beta2i error code = %d\n",err);
      exit(1);
    }
    phib2[j]=b2samp[0];
  }
}


void phiaug(double *phi,double mu, double sigma,int k){
  int l;
  /* draw n-k samples from G_0 */
  /* G_0 = N(beta0,sigmab) */
  for(l=k;l<N;l++){ 
    phi[l]=rnorm(mu,sqrt(sigma)); 
  }
} 






int main(int argc, char *argv[]){
   struct data data1; 

   struct params p;
   char *M_val;
   int i, j, k ;
   int past, np;  
   int GIBBS=10000, BURN=1000, THIN=1;
   double M, Ma, Mb;
  /* parameters */

   FILE *fp, *ft, *fV0inv, *fpri;  
   char inbuff[200];

   while ((argc>1) && (argv[1][0] == '-')){
     switch (argv[1][1]){
     case 'i': in_file= &argv[1][2];
       break; 
     case 'o': out_file= &argv[1][2];
       break;
     case 't': t_file= &argv[1][2];
         break;
     case 'M': M_val = &argv[1][2];
       M=atof(M_val);
       printf("M=%lf\n",M);
       break;
     case 'b': bsum_file = &argv[1][2]; /*  name of file for posterior 
					    means 
					    of random effects 
					    and predicted values */
       break;
     case 'p': printb= atoi(&argv[1][2]); /*  option for printing random 
					      effects */
       break;
     case 'd': np= atoi(&argv[1][2]); /* DPP if = 1 (default), par if = 0*/
       bpar=1-np;
       break;
     case 'G': GIBBS= atoi(&argv[1][2]); /* number of gibbs samples*/
       break;
     case 'B': BURN= atoi(&argv[1][2]); /* burn-in */
       break;
     case 'T': THIN= atoi(&argv[1][2]); /* thinning interval */
       break;
     case 'l': l_file= &argv[1][2]; /*  file with limits for ARMS  */
       break;
     default:
       (void)printf("Bad option %s\n",argv[1]);
     }
     argv++;
     argc--;
   }

   if ((fp = fopen(in_file, "r")) == NULL ) {
     printf( "Unable to open input data file.\n") ;
     exit( 1 );
   }
   i=0;
   /* Read in the data -- make this more robust*/
   while ( fgets(inbuff, 200, fp) != NULL) {
     /* next sscanf for the simulations */
/*     sscanf(inbuff, "%lf %lf %lf %lf %lf %lf %lf %lf %lf %lf %lf %lf %lf %lf %lf %d %d %lf\n", &data1.y[i][0], &data1.y[i][1], &data1.y[i][2], &data1.y[i][3],&data1.y[i][4], &data1.y[i][5],  &data1.y[i][6],  
 &data1.t[i][0], &data1.t[i][1], &data1.t[i][2], &data1.t[i][3],    
 &data1.t[i][4], &data1.t[i][5], &data1.t[i][6], 
 &data1.s[i], &data1.delta[i] ,&data1.obs[i], &data1.trt[i]);   
*/
/*next scanf for the data */
     sscanf(inbuff, "%lf %lf %lf %lf %lf  %lf %lf %lf %lf %lf %lf %lf %lf %d %d %lf\n", &data1.y[i][0], &data1.y[i][1], &data1.y[i][2], &data1.y[i][3],&data1.y[i][4], &data1.y[i][5],  
&data1.t[i][0], &data1.t[i][1], &data1.t[i][2], &data1.t[i][3],   
&data1.t[i][4], &data1.t[i][5], 
&data1.s[i], &data1.delta[i] ,&data1.obs[i], &data1.trt[i]);  
     i++;
   }


   if ((fV0inv = fopen("V0inv.dat", "r")) == NULL ) {
     printf( "Unable to open V0 input file.\n") ;
     exit( 1 );
   }
   if ((ft = fopen(t_file, "r")) == NULL ) {
     printf( "Unable to open times input file.\n") ;
     exit( 1 );
   }

   /* Read in the times data */
   i=0;
   while ( fgets(inbuff, 20, ft) != NULL) {
     sscanf(inbuff, "%lf\n",&p.tint[i]);  
     i++;
   }

   printf("sort survival time into vector of observation times (c[][])
      and count the number of observations before the event (pobs)\n");
   /* sort survival time into vector of observation times (c[][])
      and count the number of observations before the event (pobs) */
   for(i=0; i <N; i++){
     data1.pobs[i]=data1.obs[i];
     k=0;
     past=0;
      
    
     for(j=0;j<(data1.obs[i]);j++){       
      
      data1.c[i][j]=data1.t[i][k];
       if (data1.t[i][k]>data1.s[i] && past==0){
	 data1.c[i][j] = data1.s[i];
	 data1.pobs[i]=k;
	 past=1;
       }
       else k++;
     }
     if (data1.t[i][data1.obs[i]-1]>=data1.s[i]){
       data1.c[i][data1.obs[i]]=data1.t[i][data1.obs[i]-1];
     }
     else data1.c[i][data1.obs[i]]=data1.s[i];
     data1.pobs[i]=data1.pobs[i]+1; /* points to event time */
   }
   
   printf("setting priors and starting values\n");
   /* priors and starting values - maybe read some of these in from a file?*/   
   if ((fpri = fopen("priors.dat", "r")) == NULL ) {
     printf( "Unable to open input data file.\n") ;
     exit( 1 );
   }
   while ( fgets(inbuff, 200, fpri) != NULL) {
     /* next sscanf for the simulations */
     sscanf(inbuff, "%lf %lf %lf %lf %lf", &p.nu, &p.mug, &p.sigmagsq, &Ma, 
	    &Mb);
   }
   printf("The priors for M are %f %f\n", Ma, Mb);
   p.V0inv = gsl_matrix_calloc(3,3);
   p.S0inv = gsl_matrix_calloc(3,3);
   p.C1inv = gsl_matrix_calloc(3,3);
   gsl_matrix_set(p.C1inv,0,0,1);
   gsl_matrix_set(p.C1inv,1,1,.1);
   gsl_matrix_set(p.C1inv,2,2,.1);
   p.C0=gsl_vector_alloc(3);
   gsl_vector_set(p.C0,0,0);
   gsl_vector_set(p.C0,1,3);
   gsl_vector_set(p.C0,2,-4);
   p.C0C1i=gsl_vector_calloc(3);
   gsl_blas_dsymv(CblasUpper,1,p.C1inv,p.C0,0,p.C0C1i);  
   p.Sigma[0][0]=1;   p.Sigma[1][1]=1; p.Sigma[2][2]=1; 
   p.Sigma[1][2]=p.Sigma[2][1]=0;
   p.tau=.15, p.sigmaalpha=10, p.mualpha=1;
   gsl_matrix_fscanf (fV0inv,p.V0inv);
   gsl_matrix_memcpy(p.S0inv,p.V0inv);

   p.sigmagsq=10;  p.mug=-1; 
   p.mu0[0]=0; p.mu0[1]=0;  p.mu0[2]=0;  
   p.beta0 = .4; p.beta1=2; p.beta2=-5;
   for(i=0;i<J;i++) p.lambda[i]=1;
   p.gamma=-.5; p.alpha=0;
   p.index=0;
 
   data1.p=&p;

   /* determine interval of event */
   for (i=0;i<N;i++){
     data1.sind[i]=1;
     for (j=0;j<J;j++){
       if (data1.s[i] > p.tint[j+1]) data1.sind[i]+=1;
     }     
   }
   
   set_seed(5,72);  /* set the seed for the R random number generation functions */
   printf("calling gibbs\n");
   gibbs(&data1,M,out_file, GIBBS, BURN, THIN, Ma , Mb); 
   printf("DONE!!! \n");
}
   
void gibbs(void *d, double M, char out_file[], int GIBBS, int BURN, int THIN,
double Ma, double Mb){ 
  /* output */
  /* the next 5 lines are for the gsl random number generator */
  const gsl_rng_type * T;
  gsl_rng * r;
  gsl_rng_env_setup();
  T = gsl_rng_default;
  r = gsl_rng_alloc (T);
  int scnt;
  int zb[N];
  int i, j, l, m=10,samps=0;
  int kb[1]={1};
  double yhat[N][K],shat[N],mnyhat[N],mnyhatsq[N],shatsq[N], covys[N];
  double ltmp[J], sumg, sumgsq, ndint[J], tempsum,syhat;
  double ab=1,bb[3]={1,1,1}, scale[1], shape; /* ab and bb are 
						   hyperparameters on 
						   \tau_{beta_l} */
  double sumbisq, mu,sig;
  double a0[J],b0[1]={1},a[1];   /* a0 and b0 are  hyperparameters on 
						   \lambda_j, j=1,...,J */  
  double nclusb[N], phib0[2*N], phib1[2*N], phib2[2*N], mnb0[N],mnb1[N],mnb2[N];
  gsl_vector *betav= gsl_vector_alloc(3);

  /* ARMS parameters */
  struct data *data1;
  beta_vector b;
  int ninit = 5,npoint=50,nsamp=1,ncent=10;
  int neval, err;
  /*  alpha arms */
  double ainit[10];
  double al=-.75, ar=2.2;
  double aconvex=100;
  double asamp[1], aprev=.5;
  /*  gamma arms */
  double ginit[20], gl, gr; /* will read these in from a file */
  double gconvex=1;
  double gsamp[1], gprev;
  int dogmetrop =1, gnsamp=1;
  int gninit=5;
  /* beta0i arms  */
  double b0prev[N];
  double b0l=-4, b0r=6;
  double b0init[10]={.1,1,2,4,6};
  double b0s=.5;
  double b0samp[2];
  double b0convex=1;
  /* beta1i arms  */
  double b1prev[N];
  double b1l, b1r = 10;
  double b1init[10]={.5,1,5,7};
  double b1s=4.5;
  double b1samp[2];
  double b1convex=1;
  int dometropb1 =0;
  /* beta2i arms  */
  double b2prev[N];
  double b2l=-10, b2r=1.5;
  double b2init[10]={-4.5,-3,-2.8,-1.1,-.2};
  double b2s=-4.5;
  double b2samp[2];
  double b2convex=1;
  int b2ninit=10;
  int dometropb2 =0;
  /*  shat arms */
  double sinit[N][10], sl[N], sr[N];
  double sconvex=1,csurv=-1;
  double ssamp[N], sprev[N];
  int dosmetrop =1, snsamp=1;
  int sninit=10;
  
  /* sampling dp parameter M */
  double eta, pieta;
  double  xcent[10];
  double qcent[10]={5,10,20,30,40,50,60,70,80,95};

   int dometrop =0;
   char inbuff[200];
  
  /* hyperparameters on sigma  */
  double be[1]={.01},ae[1]={.01};
  FILE *f ,*bpost0,*bpost1,*bpost2, *bsumpost, *flim,*spost;  
  data1=d;

   if ((flim = fopen(l_file, "r")) == NULL ) {
     printf( "Unable to open input arms limits file.\n") ;
     exit( 1 );
   }

   /* Read in the limits for ARMS */
   i=0;
   while(fgets(inbuff, 80, flim) != NULL){ 
     if(i==0) sscanf(inbuff, "%lf %lf",&b0l,&b0r);
     if(i==1) sscanf(inbuff, "%lf %lf",&b1l,&b1r);
     if(i==2) sscanf(inbuff, "%lf %lf",&b2l,&b2r);
     if(i==3) sscanf(inbuff, "%lf %lf",&gl,&gr);  
     i++;
   }
   printf("%lf %lf\n%lf %lf\n%lf %lf\n%lf %lf\n",
	  b0l, b0r, b1l, b1r, b2l, b2r, gl, gr);
   gprev=(gl+gr)/2;
   for(i=0;i<N;i++){
     sl[i]=(1-data1->delta[i])*data1->s[i];
     sr[i]=data1->s[i]+6-data1->delta[i];
     for(j=1;j<11;j++){
       sinit[i][j-1]=sl[i]+(sr[i]-sl[i])/11*j;
     }
   }

  for(i=1;i<(ninit+1);i++){  
    ginit[i-1]=gl+(gr-gl)/(ninit+1)*i;
    ainit[i-1]=al+(ar-al)/(ninit+1)*i;
    b0init[i-1]=b0l+(b0r-b0l)/(ninit+1)*i;
    b1init[i-1]=b1l+(b1r-b1l)/(ninit+1)*i; 
    b2init[i-1]=b2l+(b2r-b2l)/(ninit+1)*i;  
  }      
  
  ae[0]+=.5*sumivec(data1->obs,N);
  
  for(i=0;i<N; i++){
    shat[i]=0;
    covys[j]=0;

    shatsq[i]=0;
    for(j=0;j<K;j++) yhat[i][j]=0;
  }

  /*calculate number of deaths in each interval*/
  for (i=0;i<J;i++){
    ndint[i]=0;
  }
  
  for (i=0;i<N;i++){
    ndint[data1->sind[i]-1] += data1->delta[i];
    data1->beta0i[i]=rnorm(data1->p->beta0,sqrt(1/gsl_matrix_get(data1->p->V0inv,0,0)));
    data1->beta1i[i]=rnorm(data1->p->beta1,sqrt(1/gsl_matrix_get(data1->p->V0inv,1,1)));
    data1->beta2i[i]=rnorm(data1->p->beta2,sqrt(1/gsl_matrix_get(data1->p->V0inv,2,2)));
    phib0[i]=data1->p->beta0;
    phib1[i]=data1->beta1i[i];
    phib2[i]=data1->beta2i[i];
    b2prev[i]=b2s;
    b1prev[i]=b1s;
    b0prev[i]=data1->p->beta0;
    sprev[i]=data1->s[i];
  }

  for (i=0;i<J;i++){
    a0[i]=1+ndint[i];
    printf("interval %d has %lf events, %lf %lf \n", i, ndint[i], data1->p->tint[i], data1->p->tint[i+1]);
    ltmp[i]=1;
  }
  

  /*       ********    BEGIN SAMPLING  *************    */

    kb[0]=N;  /* start with N clusters */
    for(i=0;i<N;i++){
      nclusb[i]=1;   /* one member in each cluster  */
      zb[i]=i;       /* each subject assigned to its own cluster  */
      data1->p->z[i]=i;
    }

  
  f=fopen(out_file,"w=");
  bsumpost=fopen(bsum_file,"w=");
  bpost0=fopen("bpost0.out","w=");
  bpost1=fopen("bpost1.out","w=");
  bpost2=fopen("bpost2.out","w=");

  spost=fopen("spost.out","w=");
  
  for(i=0;i<GIBBS;i++) {
    if(i%100==0)  
      printf("%d %d \n",i, kb[0]);
    if(i>BURN && i%THIN==0) samps++;
    /* ************ SAMPLE alpha ******************** */    
    if(sampalph==1){
      printf("sample alpha\n");
      err = arms(ainit,ninit,&al,&ar,alphapost,data1,&aconvex,npoint,
		 dometrop,&aprev,asamp,nsamp,qcent,xcent,ncent,&neval);      
      if(err>0){
	printf("alpha error code = %d\n",err);
	exit(1);
      }
      data1->p->alpha=asamp[0];
      aprev=asamp[0];
    }

    /*   ************* SAMPLE BETA0i ***************  */
/*     b0l=data1->p->beta0-max(10,3*sqrt(1/gsl_matrix_get(data1->p->V0inv,0,0))); */
/*     b0r=data1->p->beta0+min(10,3*sqrt(1/gsl_matrix_get(data1->p->V0inv,0,0))); */
/*     for(l=1;l<(ninit+1);l++){  */
/*       b0init[l-1]=b0l+(b0r-b0l)/(ninit+1)*l; */
/*     }     */

    /* ****SAMPLE LOCATIONS FOR CLUSTERS FOR RANDOM EFFECTS**** */

    if(bpar==0) {
      rmbeta0(phib0, b0init, ninit, b0l, b0r, data1, npoint, kb[0]) ;
      for(j=0;j<N;j++) 
	data1->beta0i[j]=phib0[zb[j]];
    }
    if(bpar==1){
      for(j=0;j<N;j++){
	data1->p->index=j;
	err = arms(b0init, ninit, &b0l, &b0r, phi0ipost, data1, 
		   &b0convex, npoint,
		   dometrop, &b0prev[j], b0samp, nsamp, qcent,
		   xcent, ncent, &neval);
      	
	if(err>0){
	  printf("beta0i error code = %d\n",err);
	  exit(1);
	}
	data1->beta0i[j]=b0samp[0];
	b0prev[j]=b0samp[0];
      }
    }

  
     
    /*   *************SAMPLE BETA1i***************  */
    b1l=data1->p->beta1-max(10,3*sqrt(1/gsl_matrix_get(data1->p->V0inv,1,1)));
    b1r=data1->p->beta1+max(10,3*sqrt(1/gsl_matrix_get(data1->p->V0inv,1,1)));
    for(l=1;l<11;l++){
      b1init[l-1]=b1l+(b1r-b1l)/11*l;  
    }
    /* ****SAMPLE LOCATIONS FOR CLUSTERS FOR RANDOM EFFECTS**** */
    if(bpar==0){
      rmbeta1(phib1, b1init, ninit, b1l, b1r, data1, npoint, kb[0]) ;
      /* next assign random effect value to appropriate location */
      for(j=0;j<N;j++) 
	data1->beta1i[j]=phib1[zb[j]];
    }

     if(bpar==1) {
       /* PARAMETRIC SAMPLING */  
       for(j=0;j<N;j++) {
	 data1->p->index=j;
	 
	 err = arms(b1init,ninit,&b1l,&b1r, phi1ipost, data1,
		    &b1convex,npoint,
		    dometropb1,&b1prev[j],b1samp,nsamp,qcent,xcent,ncent,&neval);
	 
	 if(err>0){
	   printf("beta1i error code = %d\n",err);
	   exit(1);
	 }
	 data1->beta1i[j]=b1samp[0];
	 b1prev[j]=b1samp[nsamp-1];
       }
     }
    
     
    /*   *************SAMPLE BETA2i***************  */
     /* set ARMS limits based upon current samples */
      b2l= data1->p->beta2 - 
	max(10,3*sqrt(1/gsl_matrix_get(data1->p->V0inv,2,2)));
     b2r = data1->p->beta2 + 
       max(10,3*sqrt(1/gsl_matrix_get(data1->p->V0inv,2,2)));

     for(l=1;l<11;l++){
       b2init[l-1]=b2l+(b2r-b2l)/11*l;  
     }
    /* ****SAMPLE LOCATIONS FOR CLUSTERS FOR RANDOM EFFECTS**** */
    if(bpar==0) {
      rmbeta2(phib2, b2init, b2ninit, b2l, b2r,data1,npoint, kb[0]);
      for(j=0;j<N;j++) 
	data1->beta2i[j]=phib2[zb[j]];
    }
       
    /* ********************************************** */
    

    
    /* PARAMETRIC SAMPLING */
    else if(bpar==1) {
      for(j=0;j<N;j++) {
	data1->p->index=j;
	if(b2prev[j]>b2r || b2prev[j]<b2l)  b2prev[j]=(b2r+b2l)/2;
	
	err = arms(b2init,b2ninit,&b2l,&b2r,phi2ipost,data1,&b2convex,npoint,
		   dometropb2,&b2prev[j],b2samp,nsamp,qcent,xcent,ncent,&neval); 
	if(err>0){
	  printf("beta2i error code = %d\n",err);
	  exit(1);
	}
	data1->beta2i[j]=b2samp[0];
	b2prev[j]=b2samp[nsamp-1];
      }
    }
    if(bpar==0) {

/*       if (npsample==3){ */
/* 	nogaps(zb, nclusb, phib0, phib1, phib2, data1, kb,data1->p->beta2, */
/* 	       1/data1->p->taub2, bilike, M);  */
/*       } 
      else*/ 

      if (npsample==4){
	gsl_vector_set(betav,0,data1->p->beta0);
	gsl_vector_set(betav,1,data1->p->beta1);
	gsl_vector_set(betav,2,data1->p->beta2);
	
	neal(zb, nclusb, phib0, phib1, phib2, data1, kb, 
	     betav, m, M, r); 
      }
      
      for(j=0;j<N;j++)	data1->p->z[j]=zb[j];
      
      for(j=0;j<N;j++) {
	data1->beta0i[j]=phib0[zb[j]];
	data1->beta1i[j]=phib1[zb[j]];
	data1->beta2i[j]=phib2[zb[j]];
      }
    }             
    for(j=0;j<N;j++){
      if(i>BURN && i%THIN==0) { 
	if(printb==1){
	  fprintf(bpost0,"%lf ",data1->beta0i[j]);
	  fprintf(bpost1,"%lf ",data1->beta1i[j]);
	  fprintf(bpost2,"%lf ",data1->beta2i[j]);
	}
	mnb0[j]+= data1->beta0i[j]; 
	mnb1[j]+= data1->beta1i[j]; 
	mnb2[j]+= data1->beta2i[j]; 
      }
    } 
    if(printb==1 && i>BURN && i%THIN==0){
      fprintf(bpost0, "\n");
      fprintf(bpost1, "\n");
      fprintf(bpost2, "\n");
    }


    /* ******************************************************** */
    /* ************** sample M ********************* */
    if(bpar==0) { /* only sample for nonparametric sampling */
      /* sample eta */
      eta=rbeta(M+1,N);
      /*calculate pieta*/
      pieta=((Ma+kb[0]-1)/(double)N*(Mb-log(eta)))/
	(1+((Ma+kb[0]-1)/(double)N*(Mb-log(eta))));
      if(runif(0,1)<pieta){
	M=rgamma(Ma+(double)kb[0],1/(Mb-log(eta)));
      }
      else M=rgamma(Ma+(double)kb[0]-1.0,1/(Mb-log(eta)));
    }

    /* ************ SAMPLE b_0 ******************** */
    if(bpar==1){
      for(j=0;j<N;j++){
	phib0[j]=data1->beta0i[j];
	phib1[j]=data1->beta1i[j];
	phib2[j]=data1->beta2i[j];
      }
    }
    betasamp(data1, phib0, phib1, phib2,bpar, kb[0], r);      
               
    /*sample variance of random effects */
    /* if they are assumed independent */
    /*   *************SAMPLE taub0***************  */
/*    if(samptb==1) tau0samp(data1, phib0, bpar, kb[0], ab, bb[0]);     */ 
   
    /* ********************************************** */

    /*   *************SAMPLE taub1***************  */
/*     if(samptb==1)  tau1samp(data1, phib1, bpar, kb[0], ab,bb[1]);  */
     /* ********************************************** */

    /*   *************SAMPLE taub2***************  */
   
/*    if(samptb==1) tau2samp(data1, phib2, bpar, kb[0], ab, bb[2]);  */

    /* ********************************************** */

    /*if we are using more general covariance matrix */
    /* *************SAMPLE V_0^{-1}***************  */
        gsl_vector_set(betav,0,data1->p->beta0);
        gsl_vector_set(betav,1,data1->p->beta1);
        gsl_vector_set(betav,2,data1->p->beta2);

	V0invsamp(3, kb[0], betav, phib0, phib1, phib2, data1->p->S0inv, 
		  data1->p->nu, data1->p->V0inv, r) ;

    /*   *************SAMPLE tau***************  */
    
    tempsum=0;
    for (j=0;j<N;j++){
      set_beta_vector(&b, data1->beta0i[j], data1->beta1i[j], 
		      data1->beta2i[j]);
      for (l=0;l<data1->obs[j];l++){
	tempsum+=pow(data1->y[j][l]-traj(data1->t[j][l],b),2);
      }
    }
    
    scale[0]=be[0]+.5*tempsum;     
    data1->p->tau=rgamma(ae[0],1/scale[0]);
    
    /*   **************************************  */
         
    /*   *************SAMPLE GAMMA***************  */
    err = arms(ginit,ninit,&gl,&gr, gampost, data1, &gconvex,npoint,
	       dogmetrop,&gprev,gsamp,gnsamp,qcent,xcent,ncent,&neval); 
    if(err>0){ 
      printf("gamma error code = %d\n",err); 
      exit(1); 
    } 
    
    data1->p->gamma=gsamp[0];
    gprev=gsamp[0];
    if(i>=BURN && i%THIN==0){
      sumg+=gsamp[0];
      sumgsq+=pow(gsamp[0],2);
    }

    /*   ****************************************  */

    /*   *************SAMPLE lambda***************  */
    for (j=0;j<J;j++){ 
      tempsum=0; 
      for(l=0;l<N;l++){ 	
	if(data1->sind[l] > j){
	  set_beta_vector(&b, data1->beta0i[l], data1->beta1i[l],
			  data1->beta2i[l]);
	  if(data1->p->tint[0]!=0) printf("lambda tint0 <0, i=%d\n",i);
	  tempsum+=exp(data1->p->alpha*data1->trt[l])*
	    Hs(ltmp,b,data1->p->tint,data1->p->gamma,
	       data1->pobs[l],&data1->c[l][0], j);
	}
      }
      scale[0]=b0[0]+tempsum;       
      a[0]=a0[j];     
      
      data1->p->lambda[j]= rgamma(a0[j],1/scale[0]);
      if(data1->p->lambda[j]<0){
	printf("lambda[%d] <0 at iteration %d  %lf %lf %lf %lf \n",
	       j,i, tempsum, data1->p->lambda[j],a0[j],1/scale[0]); 
	exit(1);
      }       
    }
    
    /*sample predicted Y's */
    if(spred==1){
      if(i>BURN && i%THIN ==0){
	
	for(j=0;j<N;j++){
	  set_beta_vector(&b, data1->beta0i[j], data1->beta1i[j],
			  data1->beta2i[j]);
	  syhat=0;
	  for(l=0;l<data1->obs[j];l++){
	    data1->p->yhat[j][l]=rnorm(traj(data1->t[j][l],b),
				       sqrt(1/data1->p->tau));
	    yhat[j][l]+=data1->p->yhat[j][l];
	    syhat+=data1->p->yhat[j][l];
	  }
	  mnyhat[j]+=syhat/(double)data1->obs[j];  /* mean of the 
						      predicted long. obs */
	  mnyhatsq[j]+=pow(syhat/(double)data1->obs[j],2);

	  data1->p->index=j;
	  csurv=(1-data1->delta[j])*data1->s[j];  /* = 0 if event is observed
						     = s[j] if event is censored */
	  ssamp[0]= -1;
	  scnt=0;
	  while(ssamp[0]<csurv){
	    err = arms(sinit[j],ninit,&sl[j],&sr[j], slike, data1, 
		       &sconvex,npoint,
		       dosmetrop,&sprev[j],ssamp,snsamp,qcent,xcent,
		       ncent,&neval); 
	    if(err>0){ 
	      printf("pred. s error code = %d\n",err); 
	      exit(1); 
	    } 

	    scnt++;
	    if(scnt>20){
	     printf("sampled > 20 for %d and couldn't get predicted surv. time\n"
		     ,j);
	     exit(1);
	    }
	  }
	  if(sr[j]-ssamp[0]<.05) sr[j]+=.1;
	  if(ssamp[0]-sl[j]<.05 && data1->delta[j]==1) sl[j]=max(0,(sl[j]-.05));
	  shat[j]+=log(ssamp[0]);
	  shatsq[j]+=log(ssamp[0])*log(ssamp[0]);
	  covys[j]+=log(ssamp[0])*syhat/(double)data1->obs[j];
	  sprev[j]=ssamp[0];
	  fprintf(spost,"%lf ",ssamp[0]);
      }
      fprintf(spost,"\n");
    }




}

    /*   ******  PRINT OUT POSTERIOR SAMPLES OF PARAMETERS **********  */
    if(i>BURN && i%THIN ==0){
   
    fprintf(f,"%d %11.5f %11.5f %11.5f %11.5f %11.5f %11.5f %d %11.5f   ",i,data1->p->beta0,data1->p->beta1,data1->p->beta2,data1->p->gamma,data1->p->alpha,1/data1->p->tau,
kb[0], M); 
   for(l=0;l<3;l++) for(j=0;j<(l+1);j++)fprintf(f,"%11.5f ",gsl_matrix_get(data1->p->V0inv,l,j)); 
    for(l=0;l<J;l++){
      fprintf(f,"%11.5f ",data1->p->lambda[l]);
    }
    fprintf(f,"\n");
    }
  }
  
  for(j=0;j<N;j++){
    fprintf(bsumpost,"%lf ",mnb0[j]/samps);
  }
  fprintf(bsumpost,"\n \n"); 
  for(j=0;j<N;j++){
    fprintf(bsumpost,"%f ",mnb1[j]/samps);
  }
  fprintf(bsumpost,"\n \n"); 
  for(j=0;j<N;j++){
    fprintf(bsumpost,"%f ",mnb2[j]/samps);
  }
  fprintf(bsumpost,"\n "); 
  for(j=0;j<N;j++){
    fprintf(bsumpost,"%f ",shat[j]/samps);
  }
  fprintf(bsumpost,"\n "); 
  for(j=0;j<N;j++){
    fprintf(bsumpost,"%f ",shatsq[j]/samps);
  }
  fprintf(bsumpost,"\n "); 
  
  for(j=0;j<N;j++){
    fprintf(bsumpost,"%f ",mnyhat[j]/samps);
  }
  fprintf(bsumpost,"\n "); 
  
  for(j=0;j<N;j++){
    fprintf(bsumpost,"%f ",mnyhatsq[j]/samps);
  }
  fprintf(bsumpost,"\n "); 
  
  for(j=0;j<N;j++){
    fprintf(bsumpost,"%f ",covys[j]/samps);
  }
  fprintf(bsumpost,"\n "); 
  
  for(j=0;j<N;j++){
    for(l=0;l<data1->obs[j];l++){
      fprintf(bsumpost,"%lf ",yhat[j][l]/samps);
    }
    fprintf(bsumpost,"\n "); 
  }
  fprintf(bsumpost,"\n "); 
  
   
  /* ***************  END SAMPLING  *********************  */   
  printf("bpar=%d \n",bpar);
  printf("N=%d\n",N);
  printf("sample alpha = %d\n", sampalph);
  printf("sample taub = %d\n",samptb);
  if(bpar==0) printf("M=%lf \n",M);
  printf("%d samples \n",GIBBS);
  printf("burn-in = %d  \n\n",BURN);
  printf("               mean      s.e.  \n");
  printf("gamma         %lf         %lf",sumg/samps, 
	 sqrt((sumgsq/samps-pow(sumg/samps,2))));
  printf("alpha     ");
  printf("sigma     ");
  
  for(j=0;j<J;j++){
     printf("lambda%d     ",j);
  }
}

/* Additional functions */

  double sumvec(double *x, int n){
    int i;
    double sumx;
    sumx=0;
    for (i=0;i<n;i++){
      sumx+= x[i];
    }
    return sumx;
  }

 double sumivec(int x[], int n){
    int i;
    double sumx;
    sumx=0;
    for (i=0;i<n;i++){
      sumx+= x[i];
    }
    return sumx;
  }
                
          


/* can i find a better routine to do this? */
/* should sort first? */
int samp(double *p, int n){
  double u;
  int i, elem;
  
  for (i =1; i<n; i++){
    p[i]+=p[i-1];

  }
  if(p[n-1] < (1-10^(-9))){
    printf("probabilities do not sum to 1 %14.13le\n", p[n-1]);
    exit(1);
  }
  u=runif(0,1);

  elem=0;
  while(u>p[elem]){
    elem++;
    if(elem>n){
      printf("elem>n\n");
      exit(1);
    }
  }
  return (elem);
}



double traj(double t, beta_vector b){
/* function to evaluate the trajectory at time t */
  return b.b0+b.b1*t+b.b2*t*t;
}



