#include <stdio.h>
#include <string.h>                                                          
#include <math.h>
#include <stdlib.h>
#include "arms.h"                                                            
#include <gsl/gsl_matrix.h>
#include <gsl/gsl_blas.h>
#define MATHLIB_STANDALONE 1
#include <Rmath.h>
#include <gsl/gsl_rng.h>
#include <gsl/gsl_randist.h>
#include <gsl/gsl_permute.h>
#include "global.h"

/* modified 1-23-2002 for sampling beta0-2 in clusters together */
void phisamp(gsl_matrix *phi, gsl_vector *mu, 
	     gsl_matrix *sig ,int k, int m, gsl_rng * r){
  int l, j, signum;
  double x[3];

gsl_matrix *sigtmp = gsl_matrix_alloc(3,3);
gsl_matrix_memcpy(sigtmp,sig);
  /* draw m samples from G_0 */
  /* G_0 = N(beta0,V_0) */
  for(l=k;l<(k+m);l++){ 
    mvn(mu, sigtmp, 3, x, r);
    for(j=0;j<3;j++) gsl_matrix_set(phi,l,j,x[j]);
  }
gsl_matrix_free(sigtmp);
} 


void neal(int *z, double *nclus, double *phi0, double *phi1, double *phi2, 
	  struct data *d, int *kr, gsl_vector *gmu, 
	  int m, double M, gsl_rng * r) {
  /* z = vector which indicates cluster membership  */
  gsl_matrix *gsig = gsl_matrix_calloc(3,3);
  gsl_matrix *sig = gsl_matrix_calloc(3,3);

  double  phik0, phik1, phik2;  
  int k, nxt[1], i, j, clusprev, l, descend=0, signum;
  gsl_matrix *phi = gsl_matrix_calloc(N+m,3);
  gsl_permutation * P = gsl_permutation_alloc (3);
  gsl_matrix_memcpy(gsig,d->p->V0inv);

  gsl_linalg_LU_decomp (gsig, P, &signum);
  gsl_linalg_LU_invert(gsig, P, sig);

  k = kr[0];   /* set k to number of clusters */
  for(i=0; i<k; i++){
    gsl_matrix_set(phi,i,0,phi0[i]);
    gsl_matrix_set(phi,i,1,phi1[i]);
    gsl_matrix_set(phi,i,2,phi2[i]);
  }
 
  if(runif(0,1)<.5) descend=1;
  for(l=0;l<N;l++) {
    i=l;
    if(descend==1) i=N-l-1;
    phisamp(phi, gmu, sig, k, m, r);  /* augment phi vector by m */
    nclus[z[i]]-=1;

    if(nclus[z[i]]>0) {  /* if subject i wasn't the only member of its cluster */
      /* printf("zi=%d, call newind\n", z[i]); */
      z[i]=newind(i, nclus, phi, d, k, m, M);
      if(z[i]>=k){
	/* can use copy row here from gsl */
	gsl_matrix_swap_rows(phi,k,z[i]);
/* 	phi0[k]=phi0[z[i]]; */
/* 	phi1[k]=phi1[z[i]]; */
/* 	phi2[k]=phi2[z[i]]; */
	z[i]=k;
	k++;
      }
      nclus[z[i]]+=1;
    }    
    else if(nclus[z[i]]==0){ /* if subject i was the only member of its cluster */

      /* losing a cluster k*=k-1 */
/*       nxtp[0]= 1/(double)k; */ /* probability of changing cluster */
 /*      p2=runif(0,1); */ 
/*        p2=drand48();  */
/*       nxt[0]=0; */
/*       if(p2<nxtp[0]) nxt[0]=1; */
      nxt[0]=1;
      if(nxt[0]==1){ /*  we will not be repeating cluster.  
			 therefore sample from others.
			 remove cluster by updating phi 
			 and nclus to remove the z[i] the component*/
	
	k-=1;
/* 	phik0=phi0[z[i]]; */
/* 	phik1=phi1[z[i]]; */
/* 	phik2=phi2[z[i]]; */
	
	for (j=z[i];j<k;j++){
	gsl_matrix_swap_rows(phi,j,(j+1));
/* 	  phi0[j]=phi0[j+1]; */
/* 	  phi1[j]=phi1[j+1]; */
/* 	  phi2[j]=phi2[j+1]; */
	  nclus[j]=nclus[j+1];
	}
/* 	phi0[k]=phik0;	 */
/* 	phi1[k]=phik1;	 */
/* 	phi2[k]=phik2;	 */
/* no need to do this extra copy bit because swapping rows takes care of it */
	nclus[k]=0;	
	clusprev=z[i];
	for(j=0;j<N;j++){
	  if(z[j]>clusprev) z[j]-=1;
	}
	
	z[i]=newind(i, nclus, phi, d,k, m, M );
	if(z[i]>=k){
	  gsl_matrix_swap_rows(phi,k,z[i]);
/* 	  phi0[k]=phi0[z[i]]; */
/* 	  phi1[k]=phi1[z[i]]; */
/* 	  phi2[k]=phi2[z[i]]; */
	  z[i]=k;
	  k++;
	}
	nclus[z[i]]+=1;	
      }      
      else if(nxt[0]==0){
	nclus[z[i]]=1;
      }
    }
    else if(nclus[z[i]]<0){
      printf(" nclus[z[i]]<0 \n");
      exit(1);
    }
  }
  kr[0]=k;
  for(i=0; i<k; i++){
	phi0[i]=gsl_matrix_get(phi,i,0);
	phi1[i]=gsl_matrix_get(phi,i,1);
	phi2[i]=gsl_matrix_get(phi,i,2);
  }
  gsl_matrix_free(phi);
  gsl_matrix_free(sig);
  gsl_matrix_free(gsig);

}


