#include <stdlib.h>
                                                
#include <gsl/gsl_matrix.h>
#include <gsl/gsl_linalg.h>
#include <gsl/gsl_blas.h>
#include <math.h>
#include <stdio.h>
#include <gsl/gsl_rng.h>
#include <gsl/gsl_randist.h>
#include "global.h"

#define MATHLIB_STANDALONE 1

/* This function uses the approach of Odell and Feiveson(1966) as outlined 
   by Gelfand et al(1990) to simulate from a 2x2 matrix from a wishart 
   distribution */
void wishart(double nu, gsl_matrix *S, int d, gsl_matrix *Sig, gsl_rng * r){
  /* samples a dxd matrix from a wishart(S^{-1},nu) distribution */
  /* we are passing S and will have to work with chol dec of S^{-1} */



  double y[3], tmp, tmp2;
  size_t i,j;
  int  k, Dsum, signum;
  gsl_matrix * B = gsl_matrix_alloc (d, d);
  gsl_matrix * z = gsl_matrix_alloc (d, d);
  gsl_matrix * Sinv1 = gsl_matrix_alloc (d, d); /*  */
  gsl_matrix * Sinv = gsl_matrix_alloc (d, d); /* S^{-1} */

  Dsum=d*(d+1)/2-1;
  /* S^{-1}=U'U */


/*   simulate d N~N(0,1) rvs */
  for(j=1; j<d; j++){
   for(i=0;i<(j+1);i++){
    gsl_matrix_set(z,i,j, gsl_ran_gaussian(r,1));
  }
 }
  for(i=0; i<d; i++){
    y[i]= gsl_ran_chisq(r,nu);
  }

  gsl_matrix_set(B,0,0,y[0]);

  for(j=1; j<d;j++) {
    gsl_matrix_set(B,0,j,gsl_matrix_get(z,0,j)*sqrt(y[0]));
    tmp=0;
    for(i=0;i<(j);i++) tmp+=gsl_matrix_get(z,i,j)*gsl_matrix_get(z,i,j);
    gsl_matrix_set(B,j,j,y[j]+tmp);
  }
  for(i=1;i<(d-1);i++){
    for(j=(i+1);j<d;j++) {
      tmp2=0;
      for(k=0;k<(i);k++){
	tmp2+=gsl_matrix_get(z,k,i)*gsl_matrix_get(z,k,j);
      }  /* this will not matter for d=3 */
      gsl_matrix_set(B,i, j, gsl_matrix_get(z,i,j)*sqrt(y[i])+tmp2);
}
}
  for(i=0;i<(d-1);i++)
    for(j=(i+1);j<d;j++)
    gsl_matrix_set(B,j,i,gsl_matrix_get(B,i,j));
  
  

    gsl_linalg_cholesky_decomp(S);  

    /* now calculate Sig */
    /*Sig= upper triangular of Sinv*B*lower triangular of Sinv*/

    gsl_blas_dtrmm(CblasLeft, CblasLower,CblasNoTrans,CblasNonUnit,
		   1,S,B);

    gsl_blas_dtrmm(CblasRight, CblasUpper,CblasNoTrans,CblasNonUnit,
		   1,S,B);
    gsl_matrix_memcpy(Sig,B);
    /* now Sig is our matrix! */

}

